#######################################################

#Copyright (C) IBM Corp. 2005-2017, All rights reserved.

#Author:

#Notes: 1. Use elements from MSCluster_ClusterSharedVolume class
#ClusWMI MOF file

#Revision History:

#######################################################

#Get-Module -list
#Import the FailoverClusters module into a Windows PowerShell session 

$ErrorActionPreference = "Stop"

Import-Module FailoverClusters

Echo ""
Echo ""
Echo "CLUSTER DETAILS"  
Echo ===============================================  

$CName = (Get-ClusterResource "Cluster Name" |  Get-ClusterParameter Name).Value
$CIPadd = (Get-ClusterResource "Cluster IP Address" | Get-ClusterParameter Address).Value

  Write-Host "Cluster Name: $CName " 
  Write-Host "Cluster IP Address: $CIPadd" 

Echo =============================================== 

$objs = @()

$csvs = Get-ClusterSharedVolume

Echo ""
Echo ""
Echo "Start List Of All Datapath Volumes"  
Echo ===============================================  

foreach ( $csv in $csvs )
{
   $csvinfos = $csv | select -Property Name -ExpandProperty SharedVolumeInfo
   foreach ( $csvinfo in $csvinfos )
   {
      
	$Signature = ( $csv | Get-ClusterParameter DiskSignature ).Value.substring(2) 
	
	$DiskIdGuidString = ( $csv | Get-ClusterParameter DiskIdGuid ).Value.substring(0) 
	
	if(!$Signature){
		$Signature = [int]$DiskIdGuidString
	}

	$obj = New-Object PSObject -Property @{
         Name        = $csv.Name
         CSVPath        = $csvinfo.FriendlyVolumeName
         Size        = $csvinfo.Partition.Size
         FreeSpace   = $csvinfo.Partition.FreeSpace
         UsedSpace   = $csvinfo.Partition.UsedSpace
         PercentFree = $csvinfo.Partition.PercentFree
	     SytemDisk  = ( Get-WmiObject Win32_DiskDrive | Where { "{0:x}" -f $_.Signature -eq $Signature } ).DeviceID.substring(4)
	     PhysicalDisk = ( Get-WmiObject Win32_DiskDrive | Where { "{0:x}" -f $_.Signature -eq $Signature } ).DeviceID.substring(4)
	     #SerialNumber = $csvinfo.Partition.SerialNumber
	 	
      }
	                
        ($obj).SytemDisk = $obj.SytemDisk -replace '\D+(\d+)','$1'

    	#($obj).PhysicalDisk = ($obj).PhysicalDisk -Replace "PHYSICALDRIVE", "Disk " 
	
		
      $objs += $obj
   }
}

	 $app = 'C:\Program Files\IBM\SDDDSM\datapath.exe'
	 $arg1 = 'query'
	 $arg2 = 'device'
	 $arg3 = '-a'
	 #$arg4 = ($obj).SytemDisk-1
	 
	 & $app $arg1 $arg2 $arg3
	 
	 $mpclaimstr = "mpclaim -s -d"

	 Invoke-Expression $mpclaimstr

	 Echo ""
	 Echo ""
     Echo "End List Of All Datapath Volumes"  
     Echo =============================================== 
    
    
Echo ""
Echo ""
Echo "Start List Of All Clustered Volumes(Non CSV)"  
Echo ===============================================  

Get-ClusterGroup | Get-ClusterResource | Where {$_.ResourceType.Name -eq "Physical Disk"}  


Echo ""
Echo ""
Echo "End List Of All Clustered Volumes(Non CSV)"  
Echo ===============================================  
    
	 
Echo ""
Echo ""
Echo "Start List Of All Cluster Shared Volumes(CSV)"  
Echo ===============================================  
$objs | ft -auto Name,SytemDisk,CSVPath,PhysicalDisk,SerialNumber, @{ Label = "Size(GB)" ; 
Expression = { "{0:N2}" -f ($_.Size/1024/1024/1024) } },@{ Label = "FreeSpace(GB)" ; 
Expression = { "{0:N2}" -f ($_.FreeSpace/1024/1024/1024) } },@{ Label = "UsedSpace(GB)" ; 
Expression = { "{0:N2}" -f ($_.UsedSpace/1024/1024/1024) } },@{ Label = "PercentFree" ; 
Expression = { "{0:N2}" -f ($_.PercentFree) } }
Echo ""
Echo ""
Echo "End List Of All Cluster Shared Volumes(CSV)"  
Echo ===============================================  
Echo ""
Echo ""
Echo "Start Mapping of Cluster Disks to System Physical Disks"  
Echo ========================================================  

#Cluster disk to System disk Mapping

[System.Reflection.Assembly]::LoadWithPartialName("Microsoft.Storage.Vds") | Out-Null
$VdsServiceLoader = New-Object Microsoft.Storage.Vds.ServiceLoader
$VdsService = $VdsServiceLoader.LoadService($null)
$VdsService.WaitForServiceReady()
$VdsService.Reenumerate()

#Build up a collection of all disks presented to the os
$Disks = ($VdsService.Providers |% {$_.Packs}) |% {$_.Disks}

#Import the FailoverClusters module
Import-Module FailoverClusters

#Retreve all of the CSV Luns
$AllCSVs = Get-ClusterResource
foreach ($Csv in $AllCSVs)
{
    $CSVParams = Get-ClusterParameter -InputObject $Csv
    
    #Retreve the DiskIDGuid Object from the Cluster Parameters
$DiskGUIDString = ($CSVParams | Where-object -FilterScript {$_.Name -eq "DiskIdGuid"}).Value
    
    #Match up the DiskIDs
$Disk = ($Disks | Where-Object -FilterScript {$_.DiskGuid -eq $DiskGUIDString})
        Write-Host "CSV ClusterResourceName: " $Csv.Name
        Write-Host "DiskName:                " $Disk.Name
        Write-Host
} 

$AllCSVs = Get-ClusterSharedVolume
foreach ($Csv in $AllCSVs)
{
    $CSVParams = Get-ClusterParameter -InputObject $Csv
    
    #Retreve the DiskIDGuid Object from the Cluster Parameters
$DiskGUIDString = ($CSVParams | Where-object -FilterScript {$_.Name -eq "DiskIdGuid"}).Value
    
    #Match up the DiskIDs
$Disk = ($Disks | Where-Object -FilterScript {$_.DiskGuid -eq $DiskGUIDString})
        Write-Host "CSV ClusterResourceName: " $Csv.Name        
        Write-Host "DiskName:                " $Disk.Name
        Write-Host
} 

Echo "End Mapping of Cluster Disks to System Physical Disks"  
Echo ====================================================== 